/******************************************************************************
 commands.h - et serial commands header file 
 Version: 0.9.1
 Created on: 29/06/2021
 Created by: Rohit Rajapure

 Copyright (c) 2017 Shalaka Connected Devices LLP.  All rights reserved.
 Software License Agreement

   Redistribution and use in source and binary forms, with or without
   modification, are not permitted.

   Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

   Neither the name of Shalaka Connected Devices LLP nor the names of
   its contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

#ifndef COMMANDS_H_
#define COMMANDS_H_

#include "zboss_api.h"
/******************************************************************************************
*MACROS
*
*******************************************************************************************/
// Protocol Bytes
#define SYNC                            0x55
#define RESP_ACK                        0xCC    // End of Communication
#define RESP_NACK                       0x11    // Positive Acknowledge
#define RESP_TNS                        0x33    // Negative Acknowledge
#define RESP_TIMEOUT                    0x66    // Telegram Not Supported
#define EOC                             0xF0    // End of conversation                          

#define BUFFER_SIZE                     150
#define MIN_COMMAND_LENGTH              0x04       //get command length is 4 

// Command IDs of all EURO UART Protocol Commands
#define GET_BOOTLOADER_STATE            0xD2
#define GET_RTC                         0xFC
#define GET_SWITCH_POINTS               0xFA
#define GET_SWITCH_POINT_TEMPERATUER    0xCD
#define GET_HOLIDAY_DATES               0xF8
#define GET_FLAGS                       0xF6
#define GET_TEMPERATURES                0xF4
#define GET_DEVICE_CONFIG               0xF2
#define GET_DEVICE_CONFIG_2             0xD3
#define GET_FIRMWARE_VER                0xF1
#define GET_BATTERY_STATUS              0xEF
#define GET_ENCODER_VALUE               0xEB
#define GET_POSITION                    0xE9
#define GET_SERIAL_NUMBER               0xE7
#define GET_COMBINED_FIRMWARE_VER       0xE5
#define GET_COMPOSITE_DATA1             0xE4
#define GET_PROTOCOL_BOOTLOADER_VER     0xE3
#define GET_COMPOSITE_Data2             0xE2
#define GET_PRODUCTION_MODE             0xE0
#define GET_PRODUCTION_DATA             0xD9
#define GET_DEBUG_DATA                  0xAA

#define WRITE_LCD_COMPLETE              0xEE
#define WRITE_LCD_SYMBOL                0xED
#define WRITE_LED                       0xEC
#define WRITE_LED_2                     0xD8
#define WRITE_LED_WARNING               0xD7
#define START_FIRMWARE_UPDATE           0xD1
#define TRANSFER_FIRMWARE               0xD0
#define LEAVE_BOOTLOADER_MODE           0xCF

#define SET_RTC                         0xFD
#define SET_SWITCH_POINTS               0xFB
#define SET_SWITCH_POINT_TEMPERATURE    0xCE
#define SET_HOLIDAY_DATES               0xF9
#define SET_FLAGS                       0xF7
#define SET_TEMPERATURES                0xF5
#define SET_DEVICE_CONFIG               0xF3
#define SET_DEVICE_CONFIG_2             0xD4
#define SET_POSITION                    0xEA
#define SET_SERIAL_NUMBER               0xE8
#define SET_COMBINED_FIRMWARE_VER       0xE6
#define SET_PRODUCTION_MODE             0xE1
#define SET_DISPLAY_STATE               0xD6

/// Revision der Firmware Zero
#define INTERNAL_FIRMWARE_REVISION                              'E',0,0,1
/// FLAG BYTE 1 Define
#define FLAGS_BYTE_1_AUTO_MODE				(0 << 0)
#define FLAGS_BYTE_1_MANU_MODE				(1 << 0)
#define FLAGS_BYTE_1_SAVING_MODE			(0 << 1)
#define FLAGS_BYTE_1_HEATING_MODE			(1 << 1)
#define FLAGS_BYTE_1_KEY_PROTECTION_ACTIVE	(1 << 2)
#define FLAGS_BYTE_1_INT_WINDOW_TRIGGERD	(1 << 3)
#define FLAGS_BYTE_1_EXT_WINDOW_ACTIVATE	(1 << 4)
#define FLAGS_BYTE_1_RTC_INVALID			(1 << 5)
#define FLAGS_BYTE_1_BATTERY_WEAK			(1 << 6)
#define FLAGS_BYTE_1_HOLIDAY_ACTIVE			(1 << 7)
/// FLAG BYTE 2 Define
#define FLAGS_BYTE_2_ENCODER_BIT			(1 << 0)
#define FLAGS_BYTE_2_SETPOINT_CHANGED		(1 << 1)
#define FLAGS_BYTE_2_MASTER_RESET_REQUEST	(1 << 2)
#define FLAGS_BYTE_2_MEASURED_TEMP_CHANGED	(1 << 3)
#define FLAGS_BYTE_2_MASTER_RESET_SUCCESS	(1 << 4)
#define FLAGS_BYTE_2_BOOST_MODE                 (1 << 5)
#define FLAGS_BYTE_2_PRODUCTION_MODE		(1 << 6)
#define FLAGS_BYTE_2_VALVE_POSITION_CHANGED	(1 << 7)
//#define RESERVED
//#define RESERVED
/// FLAG BYTE 3 Define
#define FLAG_BYTE_3_ALL_BUTTON				(0xFF)
#define FLAG_BYTE_3_NO_BUTTON				(0)
/// FLAG BYTE 4 Define
#define FLAGS_BYTE_4_ADAP_SUCCESSFUL		(0)
#define FLAGS_BYTE_4_ERROR_E1				(1)
#define FLAGS_BYTE_4_ERROR_E2				(2)
#define FLAGS_BYTE_4_ERROR_E3				(3)
#define FLAGS_BYTE_4_PREP					(4)
#define FLAGS_BYTE_4_INST					(5)
#define FLAGS_BYTE_4_ADAP					(6)
//#define RESERVED							(7 << 0)
#define FLAGS_BYTE_4_ADAP_REQUEST			(1 << 3)
#define FLAGS_BYTE_4_GO_TO_MOUNTING_POS		(1 << 4)
//#define RESERVED		(1 << 5)
//#define RESERVED		(1 << 6)
//#define RESERVED		(1 << 7)
/// FLAG BYTE 5 Define
#define FLAGS_BYTE_5_PAIR_REQUEST			(1 << 0)
#define FLAGS_BYTE_5_UNPAIR_REQUEST			(1 << 1)
#define FLAGS_BYTE_5_UN_PAIR_SUCCESS		(1 << 2)
#define FLAGS_BYTE_5_UN_PAIR_ERROR			(1 << 3)
#define FLAGS_BYTE_5_UN_PAIR_TIMEOUT		(1 << 4)
#define FLAGS_BYTE_5_RF_COMMUNICATION_OK	(1 << 5)
#define FLAGS_BYTE_5_DEVICE_PAIRED			(1 << 6)
//#define RESERVED		(1 << 6)
//#define RESERVED		(1 << 7)

//Device Config
#define DEVICE_CONFIG_BYTE_3_INTERNAL_WINDOW_OPEN		(1 << 0)
#define DEVICE_CONFIG_BYTE_3_TEMPERATURE_CONTROLLER		(0 << 2)
#define DEVICE_CONFIG_BYTE_3_VALVE_CONTROLLER			(1 << 2)
#define DEVICE_CONFIG_BYTE_3_LCD_ROTATION				(1 << 4)

#define PRODUCTION_MODE_INACTIVE			(0 << 0)
#define PRODUCTION_MODE_ACTIVE				(1 << 0)
/****************************************************************************/
/***        Macro Definitions                                             ***/
/****************************************************************************/
#define LCD_RF_SYMBOL 	0x01
#define LCD_SYMBOL_ON 	0xFF
#define LCD_SYMBOL_OFF 	0x00

#define TRV_MODE_AUTO 			0x00
#define TRV_MODE_POSITIONING	        0x01
#define TRV_MODE_MANU			0x02

#define START_EXTERNAL_WINDOW_OPEN			0x20
#define STOP_EXTERNAL_WINDOW_OPEN			0x20
#define START_INTERNAL_WINDOW_OPEN			0x10
#define STOP_INTERNAL_WINDOW_OPEN			0x10
#define ACTIVATE_WINDOW_OPEN				0x10
/******************************************************************************************
*TYPEDEFS
*
*******************************************************************************************/

/******************************************************************************************
*FUNCTIONS
*
*******************************************************************************************/
uint8_t GetMessageIdResponse(uint8_t ui8MessageId);
void CommandBootloader (void);
/******************************************************************************************
*CRC generation function
*return CRC
*******************************************************************************************/
uint8_t ui8GenerateCheckSum(uint8_t buffer[],uint8_t ui8Count);
uint8_t SetFlags();
uint8_t SendSetTemperature();

//testing 
void ReadRemainigBatteryPercentage(zb_uint8_t ui8BatteryPercentage);
void vCalculateBattery(uint8_t u8BatteryInPercent) ;

void localTemperatureValue(zb_uint16_t ui16local_tempvalue);
void CurrentSetpointChangeValue(zb_uint16_t ui16CurrentSetpoint);
void OccupiedHeatingSetpoint(zb_uint16_t ui16OccupiedSetpoint);
void ReadOccupiedHeatingSetpoint(zb_uint16_t ui16ReadOccupiedSetpoint);
void UnOccupiedHeatingSetpoint(zb_uint16_t ui16UnOccupiedSetpoint);

void SetValvePosition(zb_uint8_t ui8ValvePosition);
void ReadErrorStatus(zb_uint8_t ui8ErrorStatus);
void HostFlagStatus(zb_uint32_t ui32HostFlagsStatus);
void SetTRVMode (zb_uint8_t ui8TRVMode);
/****************************************************************************
 *
 * NAME: UARTStateSendFlags
 *
 * DESCRIPTION:
 * send flag (set flag) when Host attribute changed 
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendFlags(void);

/****************************************************************************
 *
 * NAME: UARTStateSendFlagsAuto
 *
 * DESCRIPTION:
 * Flags Device Configuration synchronisieren
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendFlagsAuto(void);
/****************************************************************************
 *
 * NAME: UARTStateSendFlagsManu
 *
 * DESCRIPTION:
 * Flags nach Device Configuration synchronisieren
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendFlagsManu(void);
/****************************************************************************
 *
 * NAME: UARTStateSendDeviceConfig
 *
 * DESCRIPTION:
 * Device config senden
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendDeviceConfig(void);
/****************************************************************************
 *
 * NAME: UARTStateSendDeviceConfig2
 *
 * DESCRIPTION:
 * Device config send
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendDeviceConfig2(void);
/****************************************************************************
 *
 * NAME: UARTStateSendFlagsLeave
 *
 * DESCRIPTION:
 * RF Symbol reset
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendFlagsLeave(void);
/****************************************************************************
 *
 * NAME: UARTStateSendFlagsRFCommunicationOK
 *
 * DESCRIPTION:
 * RF Symbol reset
 *
 * RETURNS:
 * void
 *
 *****************************************************************************/
void UARTStateSendFlagsRFCommunicationOK(void);
/****************************************************************************
 *
 * NAME: UARTStateSendFlagsRFCommunicationLost
 *
 * DESCRIPTION:
 * RF Symbol reset
 *
 * RETURNS:
 * void
 *
 *****************************************************************************/
void UARTStateSendFlagsRFCommunicationLost(void);
/****************************************************************************
 *
 * NAME: UARTStateSendFlagsJoined
 *
 * DESCRIPTION:
 * Flags senden
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendFlagsJoined(void);  
/****************************************************************************
 *
 * NAME: UARTStateMasterResetRequest
 *
 * DESCRIPTION:
 * Flags send with Master Reset Request
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateMasterResetRequest(void);
/****************************************************************************
 *
 * NAME: UARTStateMasterResetSuccess
 *
 * DESCRIPTION:
 * Flags send with Master Reset Success
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateMasterResetSuccess(void);
/****************************************************************************
 *
 * NAME: UARTStateSendSetProductionModePass
 *
 * DESCRIPTION:
 * Send Productions mode passed
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendSetProductionModePass(void);
/****************************************************************************
 *
 * NAME: UARTStateSendStartProductionMode
 *
 * DESCRIPTION:
 * Activate production mode
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendStartProductionMode(void);
/****************************************************************************
 *
 * NAME: UARTStateSendRTC
 *
 * DESCRIPTION:
 * send rtc
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendRTC(void);
/****************************************************************************
 *
 * NAME: UARTStateSendValvePosition
 *
 * DESCRIPTION:
 * Send valve position
 *
 * RETURNS:
 * void
 *
 ****************************************************************************/
void UARTStateSendValvePosition(void);

/****************************************************************************
 *
 * NAME: UARTStateStartFirmwareUpdate
 *
 * DESCRIPTION:
 * Host Firmware Update start
 *
 * RETURNS:
 * void
 *
****************************************************************************/
void UARTStateStartFirmwareUpdate(void);

/****************************************************************************
 *
 * NAME: UARTStateTransferFirmwareUpdate
 *
 * DESCRIPTION:
 * Host transfer Firmware 
 *
 * RETURNS:
 * void
 *
****************************************************************************/
void UARTStateTransferFirmwareUpdate(void);

  
#endif /* COMMANDS_H_ */